import 'api_endpoint.dart';
import 'api_request_method.dart';
import 'api_response.dart';
import 'dto/transfer_object.dart';

class ApiBase {
  final String _baseUrl;
  final List<ApiEndpoint> _endpoints;

  ApiBase(this._baseUrl, this._endpoints);

  Map<String, String> getRequestHeaders() {
    //TODO auth token and stuff
    return {};
  }

  //TODO body data / query data / header data ?
  Future<ApiResponse<TResult>> get<TResult>(ApiEndpoint<TResult> endpoint,
          [Map<String, dynamic>? params]) async =>
      await request(ApiRequestMethod.GET, endpoint, params);

  Future<ApiResponse<TResult>> post<TResult>(ApiEndpoint<TResult> endpoint, TransferObject data,
          [Map<String, dynamic>? params]) async =>
      await request(ApiRequestMethod.POST, endpoint, params, data);

  Future<ApiResponse<TResult>> put<TResult>(ApiEndpoint<TResult> endpoint, TransferObject data,
          [Map<String, dynamic>? params]) async =>
      await request(ApiRequestMethod.PUT, endpoint, params, data);

  Future<ApiResponse<TResult>> patch<TResult>(ApiEndpoint<TResult> endpoint, TransferObject data,
          [Map<String, dynamic>? params]) async =>
      await request(ApiRequestMethod.PATCH, endpoint, params, data);

  Future<ApiResponse<TResult>> delete<TResult>(ApiEndpoint<TResult> endpoint,
          [Map<String, dynamic>? params]) async =>
      await request(ApiRequestMethod.DELETE, endpoint, params);

  Future<ApiResponse<TResult>> request<TResult>(ApiRequestMethod method,
      ApiEndpoint<TResult> endpoint, Map<String, dynamic>? params, [TransferObject? data]) async {
    return await endpoint.request(_baseUrl, params, method,
        headers: getRequestHeaders(), body: data);
  }
}
